@@ -326,9 +326,15 @@ class TypeDeductor(BaseModule):
             )
 
             if stats_v2[col_name]['identifier'] is not None:
+
                 if col_name not in self.transaction.lmd['force_column_usage']:
                     if col_name not in self.transaction.lmd['predict_columns']:
-                        self.transaction.lmd['columns_to_ignore'].append(col_name)
+                        if (self.transaction.lmd.get('tss', None) and
+                                self.transaction.lmd['tss']['is_timeseries'] and
+                                col_name in self.transaction.lmd['tss']['order_by']):
+                            pass
+                        else:
+                            self.transaction.lmd['columns_to_ignore'].append(col_name)
 
             if data_subtype_dist:
                 self.log.info(f'Data distribution for column "{col_name}" '
